from giara.constants import APP_ID, APP_NAME, PROFILE
from gi.repository import Gtk, Adw
from giara.confManager import ConfManager
from giara.main_ui import MainUI
from giara.accel_manager import add_accelerators


class AppWindow(Adw.ApplicationWindow):
    def __init__(self, **kwargs):
        super().__init__()
        self.confman = ConfManager()
        self.reddit = self.confman.reddit

        self.set_title(APP_NAME)
        self.set_icon_name(APP_ID+'.dev' if PROFILE == 'development' else '')

        self.main_box = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)

        self.main_ui = MainUI()
        self.main_box.append(self.main_ui)

        self.set_content(self.main_box)

        def toggle_popover(*args):
            self.main_ui.deck.left_stack.get_headerbar().menu_btn.popup()
            # if popover.is_visible():
            #     popover.popdown()
            # else:
            #     popover.popup()

        add_accelerators(
            self,
            [
                {
                    'combo': 'F10',
                    'cb': toggle_popover
                }
            ]
        )

        self.set_default_size(
            self.confman.conf['windowsize']['width'],
            self.confman.conf['windowsize']['height']
        )

        self.confman.connect('dark_mode_changed', self.on_dark_mode_changed)

    def present(self, *args, **kwargs):
        super().present(*args, **kwargs)
        self.on_dark_mode_changed()

    def on_dark_mode_changed(self, *args):
        Adw.StyleManager.get_default().set_color_scheme(
            Adw.ColorScheme.FORCE_DARK if self.confman.conf['dark_mode']
            else Adw.ColorScheme.DEFAULT
        )
        # Gtk.Settings.get_default().set_property(
        #     'gtk-application-prefer-dark-theme',
        #     self.confman.conf['dark_mode']
        # )

    def emit_destroy(self, *args):
        self.emit('destroy')

    def on_destroy(self, *args):
        alloc = self.get_allocation()
        self.confman.conf['windowsize'] = {
            'width': alloc.width,
            'height': alloc.height
        }
        self.hide()
        self.confman.save_conf()

    def do_startup(self):
        pass
