from gettext import gettext as _
from gi.repository import Gtk, Pango
from praw.models import Comment, Submission, Message
from giara.custom_emoji import CustomEmoji
import re


EMOJI_REGEX = re.compile(r':[a-zA-Z0-9_\-]+:')


class FlairLabel(Gtk.Box):
    styles = list()

    @classmethod
    def new_from_type(cls, item):
        if isinstance(item, Comment):
            return cls(
                _('Comment'), '#3584e4', 'light'
            )
        elif isinstance(item, Submission):
            return cls(
                _('Post'), '#3584e4', 'light'
            )
        elif isinstance(item, Message):
            return cls(
                _('Message'), '#3584e4', 'light'
            )
        else:
            raise TypeError(
                'Cannot create flair for unknown type '+type(item)
            )

    @classmethod
    def new_from_post(cls, post):
        return cls(
            post.link_flair_text,
            post.link_flair_background_color,
            post.link_flair_text_color,
            subreddit=post.subreddit
        )

    @classmethod
    def new_type_image(cls):
        return cls(
            _('Image'), '#33d17a', 'dark'
        )

    @classmethod
    def new_type_video(cls):
        return cls(
            _('Video'), '#f6d32d', 'dark'
        )

    @classmethod
    def new_type_text(cls):
        return cls(
            _('Text'), '#986a44', 'light'
        )

    @classmethod
    def new_type_link(cls):
        return cls(
            _('Link'), '#3584e4', 'light'
        )

    @classmethod
    def new_nsfw(cls):
        return cls(
            _('NSFW'), '#e01b24', 'light'
        )

    def __init__(self, text, bg, fg, subreddit=None):
        super().__init__(hexpand=False, orientation=Gtk.Orientation.HORIZONTAL)

        if subreddit is not None:
            emoji_match = EMOJI_REGEX.search(text)
            while emoji_match is not None:
                emoji = text[emoji_match.start():emoji_match.end()]
                text = text.replace(emoji, '').strip()
                self.custom_emoji = CustomEmoji(subreddit, emoji)
                self.append(self.custom_emoji)
                emoji_match = EMOJI_REGEX.search(text)

        self.label = Gtk.Label(
            label=text, halign=Gtk.Align.START, wrap=True,
            wrap_mode=Pango.WrapMode.WORD_CHAR
        )
        self.append(self.label)
        style_context = self.get_style_context()
        style_context.add_class('flair')
        if fg and bg:
            flair_name = (fg+bg).replace('#', '0x')
            if flair_name not in FlairLabel.styles:
                FlairLabel.styles.append(flair_name)
                css = f'''.flair-{flair_name} {{
                    background-color: {bg};
                    color: {"white" if fg == "light" else "black"};
                }}'''
                css_provider = Gtk.CssProvider()
                css_provider.load_from_data(css.encode())
                Gtk.StyleContext().add_provider_for_display(
                    self.get_display(), css_provider,
                    Gtk.STYLE_PROVIDER_PRIORITY_USER
                )
            style_context.add_class(f'flair-{flair_name}')
